#include "NormalMapping.h"

/* STATIC VARIABLES */
CGprogram *NormalMapping::cgVertexProgram;
CGprogram *NormalMapping::cgFragmentProgram;

CGprofile *NormalMapping::cgVertexProfile;
CGprofile *NormalMapping::cgFragmentProfile;

CGparameter NormalMapping::cgModelViewProj;
CGparameter NormalMapping::cgModelViewIT;
CGparameter NormalMapping::cgLightPosition;
CGparameter NormalMapping::cgNormalMap;

CGcontext NormalMapping::cgContext;

GLuint NormalMapping::normalMapID;

Vector3f *NormalMapping::lightPosition = NULL;

NormalMapping::NormalMapping(Vector3f *lightPosition)
{
	/* Initialize Shaders */
	cgVertexProgram = new CGprogram;
	cgFragmentProgram = new CGprogram;
	cgVertexProfile = new CGprofile;
	cgFragmentProfile = new CGprofile;

	loadNormalMap();

	initCg(cgVertexProfile, cgFragmentProfile, cgVertexProgram, cgFragmentProgram);

	NormalMapping::lightPosition = lightPosition;
}

NormalMapping::~NormalMapping()
{
	/* Demonstrate proper deallocation of Cg runtime data structures.
	 * Not strictly necessary if we are simply going to exit. */
	cgDestroyProgram(*cgVertexProgram);
	cgDestroyProgram(*cgFragmentProgram);
	//cgDestroyContext(*cgContext);
}

void NormalMapping::enableShader()
{
	cgGLBindProgram(*cgVertexProgram);
	cgGLEnableProfile(*cgVertexProfile);
	cgGLBindProgram(*cgFragmentProgram);
	cgGLEnableProfile(*cgFragmentProfile);
}

void NormalMapping::disableShader()
{
	cgGLDisableProfile(*cgFragmentProfile);
	cgGLDisableProfile(*cgVertexProfile);
}

void NormalMapping::loadNormalMap()
{
	normalMapID = ilutGLLoadImage("../../Shader/NormalMapping/resource/normalmaps/playerNormals.png");
	if (normalMapID == 0)
	{
		printf("[ERROR] Normalmap not found!\n");
		exit(-1);
	}
}

void NormalMapping::cgErrorCallback()
{
	CGerror lastError = cgGetError();
	if(lastError)
	{
		printf("%s\n\n", cgGetErrorString(lastError));
		printf("%s\n", cgGetLastListing(cgContext));
		printf("Cg error, exiting...\n");

		exit(-1);
	}
}

void NormalMapping::initCg(CGprofile *cgVertexProfile, CGprofile *cgFragmentProfile,
						CGprogram *cgVertexProgram, CGprogram *cgFragmentProgram)
{
	cgSetErrorCallback(cgErrorCallback);
	cgContext = cgCreateContext();

	*cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(*cgVertexProfile);
	*cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(*cgFragmentProfile);

	*cgVertexProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/NormalMapping/resource/cg/vp_normalMapping.cg",
		*cgVertexProfile,
		"vp_normalMapping", 
		NULL );

	*cgFragmentProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/NormalMapping/resource/cg/fp_normalMapping.cg",
		*cgFragmentProfile,
		"fp_normalMapping", 
		NULL );


	cgGLLoadProgram(*cgVertexProgram);
	cgGLLoadProgram(*cgFragmentProgram);


	cgModelViewProj		= cgGetNamedParameter(*cgVertexProgram, "modelViewProj");
	cgModelViewIT		= cgGetNamedParameter(*cgVertexProgram, "modelViewIT");
	cgLightPosition     = cgGetNamedParameter(*cgVertexProgram, "lightPosition");
	cgNormalMap			= cgGetNamedParameter(*cgFragmentProgram, "normalMap");

	if(!cgModelViewProj ||
		!cgModelViewIT ||
		!cgLightPosition ||
		!cgNormalMap)
	{
		printf("Unable to retrieve program parameters, exiting...\n");
		exit(0);
	}
}

void NormalMapping::setShaderParameters()
{	
	glBindTexture(GL_TEXTURE_2D, normalMapID);
	cgGLSetStateMatrixParameter(cgModelViewProj,
								CG_GL_MODELVIEW_PROJECTION_MATRIX,
								CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelViewIT,
		                        CG_GL_MODELVIEW_MATRIX,
								CG_GL_MATRIX_INVERSE);
	cgGLSetTextureParameter(cgNormalMap,
							normalMapID);

	cgGLSetParameter3f(cgLightPosition, lightPosition->x(), lightPosition->y(), lightPosition->z());
}
