#include "Refraction.h"

/* STATIC VARIABLES */
CGprogram *Refraction::cgVertexProgram;
CGprogram *Refraction::cgFragmentProgram;
CGprofile *Refraction::cgVertexProfile;
CGprofile *Refraction::cgFragmentProfile;

CGparameter Refraction::cgModelViewProj;
CGparameter Refraction::cgModelViewIT;
CGparameter Refraction::cgModelView;
CGparameter Refraction::cgEyePositionV;
CGparameter Refraction::cgEyePositionF;
CGparameter Refraction::cgLightPosition;
CGparameter Refraction::cgEnvironmentMap;
//CGparameter Refraction::cgNormalMap;

CGcontext Refraction::cgContext;

GLuint Refraction::cubemapTextureID;
//GLuint Refraction::normalMapID;

char Refraction::buffer[1024];

int Refraction::cubeSides[] = { GL_TEXTURE_CUBE_MAP_POSITIVE_X_ARB, 
								GL_TEXTURE_CUBE_MAP_NEGATIVE_X_ARB,
								GL_TEXTURE_CUBE_MAP_POSITIVE_Y_ARB,
								GL_TEXTURE_CUBE_MAP_NEGATIVE_Y_ARB,
								GL_TEXTURE_CUBE_MAP_POSITIVE_Z_ARB,
								GL_TEXTURE_CUBE_MAP_NEGATIVE_Z_ARB };



Refraction::Refraction()
{
	/* Initialize Shaders */
	cgVertexProgram = new CGprogram;
	cgFragmentProgram = new CGprogram;
	cgVertexProfile = new CGprofile;
	cgFragmentProfile = new CGprofile;

	if(!GLEW_ARB_texture_cube_map)
	{
		printf("GL_ARB_texture_cube_map extension is not supported\n");
		exit(-1);
	}

	//ilInit();

	loadCubeMap();

	//loadNormalMap();

	initCg(cgVertexProfile, cgFragmentProfile, cgVertexProgram, cgFragmentProgram);
}

Refraction::~Refraction(void)
{
	/* Demonstrate proper deallocation of Cg runtime data structures.
	 * Not strictly necessary if we are simply going to exit. */
	cgDestroyProgram(*cgVertexProgram);
	cgDestroyProgram(*cgFragmentProgram);
	//cgDestroyContext(*cgContext);
}

void Refraction::enableShader(void)
{
	cgGLBindProgram(*cgVertexProgram);
	cgGLEnableProfile(*cgVertexProfile);
	cgGLBindProgram(*cgFragmentProgram);
	cgGLEnableProfile(*cgFragmentProfile);
}

void Refraction::disableShader(void)
{
	cgGLDisableProfile(*cgFragmentProfile);
	cgGLDisableProfile(*cgVertexProfile);
}

void Refraction::loadCubeMap(void)
{
	glGenTextures(1, &cubemapTextureID);
	glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubemapTextureID);

	string *name = new string;
	*name = ("cube");
	//printf("Cubemap Texture locations:\n");
	for(int i = 0; i <= 5; i++)
	{
		sprintf(buffer, "../../Shader/Refraction/resource/cubemaps/%s_%i.bmp", name->c_str(), i);
		//printf("%s\n", buffer);

		ilLoadImage(buffer);

		ilConvertImage(IL_RGB, IL_UNSIGNED_BYTE);
		gluBuild2DMipmaps(cubeSides[i],
			GL_RGB8, 
			(ILfloat)ilGetInteger(IL_IMAGE_WIDTH),
			(ILfloat)ilGetInteger(IL_IMAGE_HEIGHT),
			GL_RGB,
			GL_UNSIGNED_BYTE,
			ilGetData());
	}
}

//void Refraction::loadNormalMap(void)
//{
//	normalMapID = ilutGLLoadImage("../../Shader/Refraction/resource/normalmaps/NM_face.dds");
//	if (normalMapID == 0)
//	{
//		printf("[ERROR] Normalmap not found!\n");
//		exit(-1);
//	}
//}

void Refraction::cgErrorCallback(void)
{
	CGerror lastError = cgGetError();
	if(lastError)
	{
		printf("%s\n\n", cgGetErrorString(lastError));
		printf("%s\n", cgGetLastListing(cgContext));
		printf("Cg error, exiting...\n");

		exit(-1);
	}
}

void Refraction::initCg(CGprofile *cgVertexProfile, CGprofile *cgFragmentProfile,
						CGprogram *cgVertexProgram, CGprogram *cgFragmentProgram)
{
	cgSetErrorCallback(cgErrorCallback);
	cgContext = cgCreateContext();

	*cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(*cgVertexProfile);
	*cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(*cgFragmentProfile);

	*cgVertexProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/Refraction/resource/cg/vp_refraction.cg",
		*cgVertexProfile,
		"vp_refraction", 
		NULL );

	*cgFragmentProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/Refraction/resource/cg/fp_refraction.cg",
		*cgFragmentProfile,
		"fp_refraction", 
		NULL );


	cgGLLoadProgram(*cgVertexProgram);
	cgGLLoadProgram(*cgFragmentProgram);


	cgModelViewProj		= cgGetNamedParameter(*cgVertexProgram, "modelViewProj");
	cgModelView			= cgGetNamedParameter(*cgVertexProgram, "modelView");
	cgModelViewIT   	= cgGetNamedParameter(*cgVertexProgram, "modelViewIT");	
	cgEyePositionV       = cgGetNamedParameter(*cgVertexProgram, "eyePosition");
	cgEyePositionF       = cgGetNamedParameter(*cgFragmentProgram, "eyePosition");
	cgLightPosition     = cgGetNamedParameter(*cgFragmentProgram, "lightPosition");
	cgEnvironmentMap    = cgGetNamedParameter(*cgFragmentProgram, "environmentMap");
	//cgNormalMap			= cgGetNamedParameter(*cgFragmentProgram, "normalMap");

	if(!cgModelViewProj ||
		!cgModelView ||
		!cgModelViewIT ||
		!cgEyePositionV ||
		!cgEyePositionF ||
		!cgLightPosition ||
		!cgEnvironmentMap) //||cgNormalMap
	{
		printf("Unable to retrieve program parameters, exiting...\n");
		exit(0);
	}
}

void Refraction::setShaderParameters(Vector3f *lightPosition, Vector3f cameraPosition)
{	
	cgGLSetStateMatrixParameter(cgModelViewProj,
								CG_GL_MODELVIEW_PROJECTION_MATRIX,
								CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelView,
								CG_GL_MODELVIEW_MATRIX,
								CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelViewIT,
								CG_GL_MODELVIEW_MATRIX,
								CG_GL_MATRIX_INVERSE_TRANSPOSE);

	cgGLSetTextureParameter(cgEnvironmentMap,
							cubemapTextureID);
	/*cgGLSetTextureParameter(cgNormalMap,
							normalMapID);*/

	cgGLSetParameter3f(cgEyePositionV, cameraPosition.x(), cameraPosition.y(), cameraPosition.z());
	cgGLSetParameter3f(cgEyePositionF, cameraPosition.x(), cameraPosition.y(), cameraPosition.z());
	cgGLSetParameter3f(cgLightPosition, lightPosition->x(), lightPosition->y(), lightPosition->z());
}

//void Refraction::updateCubeMap(void)
//{
//	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
//    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
//    glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
//	/*glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
//	glTexParameteri(GL_TEXTURE_CUBE_MAP_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);*/
//}

void Refraction::drawCubeMap(void)
{
	//glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, cubemapTextureID);
	// set cubemap texture parameters
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_CUBE_MAP, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

	GLfloat s_plane[] = {1.0, 0.0, 0.0, 0.0};
	GLfloat t_plane[] = {0.0, 1.0, 0.0, 0.0};
	GLfloat r_plane[] = {0.0, 0.0, 1.0, 0.0};

	//cubeMapContainer.loadIdentity();
	//glGetFloatv(GL_MODELVIEW_MATRIX, cubeMapContainer.array());
	glTexGenfv(GL_S, GL_OBJECT_PLANE, s_plane);
	glTexGenfv(GL_T, GL_OBJECT_PLANE, t_plane);
	glTexGenfv(GL_R, GL_OBJECT_PLANE, r_plane);

	// Initialize object linear texture generation
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
	//glTexGeni (GL_S, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
	//glTexGeni (GL_T, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
	//glTexGeni (GL_R, GL_TEXTURE_GEN_MODE, GL_NORMAL_MAP);
	//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
	//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
	//glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP_EXT);
	
	glPushAttrib (GL_ENABLE_BIT | GL_POLYGON_BIT | GL_LIGHTING_BIT);
		glDisable (GL_LIGHTING);
		glDisable (GL_CULL_FACE);

		glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glEnable(GL_TEXTURE_GEN_R);
		glEnable(GL_TEXTURE_CUBE_MAP_ARB);

		//glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

		glMatrixMode(GL_MODELVIEW);
	
		glutSolidCube(15.0);

		glDisable(GL_TEXTURE_CUBE_MAP_ARB);
		glDisable(GL_TEXTURE_GEN_R);
		glDisable(GL_TEXTURE_GEN_T);
		glDisable(GL_TEXTURE_GEN_S);
		glEnable(GL_CULL_FACE);
	glPopAttrib();
}