#include "ShadowMapping.h"

/* STATIC VARIABLES */
CGprogram *ShadowMapping::cgVertexProgram;
CGprogram *ShadowMapping::cgFragmentProgram;
CGprofile *ShadowMapping::cgVertexProfile;
CGprofile *ShadowMapping::cgFragmentProfile;
CGparameter ShadowMapping::cgModelViewProj;
CGparameter ShadowMapping::cgModelViewIT;
CGparameter ShadowMapping::cgModelView;
CGparameter ShadowMapping::cgEyePosition;
CGparameter ShadowMapping::cgLightPosition;


CGcontext ShadowMapping::cgContext;

char ShadowMapping::buffer[1024];



ShadowMapping::ShadowMapping(GLuint shadowMapTexture)
{
	/* Initialize Shaders */
	cgVertexProgram = new CGprogram;
	cgFragmentProgram = new CGprogram;
	cgVertexProfile = new CGprofile;
	cgFragmentProfile = new CGprofile;

	loadShadowMap(shadowMapTexture);


	initCg(cgVertexProfile, cgFragmentProfile, cgVertexProgram, cgFragmentProgram);
}

ShadowMapping::~ShadowMapping(void)
{
	cgDestroyProgram(*cgVertexProgram);
	cgDestroyProgram(*cgFragmentProgram);
	cgDestroyContext(cgContext);
}

void ShadowMapping::enableShader(void)
{
	cgGLBindProgram(*cgVertexProgram);
	cgGLEnableProfile(*cgVertexProfile);
	cgGLBindProgram(*cgFragmentProgram);
	cgGLEnableProfile(*cgFragmentProfile);
}

void ShadowMapping::disableShader(void)
{
	cgGLDisableProfile(*cgFragmentProfile);
	cgGLDisableProfile(*cgVertexProfile);
}

void ShadowMapping::loadShadowMap(GLuint shadowMapTexture)
{
	glGenTextures(1, &shadowMapTexture);
	glBindTexture(GL_TEXTURE_2D, shadowMapTexture);
}

void ShadowMapping::cgErrorCallback(void)
{
	CGerror lastError = cgGetError();
	if(lastError)
	{
		printf("%s\n\n", cgGetErrorString(lastError));
		printf("%s\n", cgGetLastListing(cgContext));
		printf("Cg error, exiting...\n");

		exit(-1);
	}
}

void ShadowMapping::initCg(CGprofile *cgVertexProfile, CGprofile *cgFragmentProfile,
						   CGprogram *cgVertexProgram, CGprogram *cgFragmentProgram)
{
	cgSetErrorCallback(cgErrorCallback);
	cgContext = cgCreateContext();

	*cgVertexProfile = cgGLGetLatestProfile(CG_GL_VERTEX);
	cgGLSetOptimalOptions(*cgVertexProfile);
	*cgFragmentProfile = cgGLGetLatestProfile(CG_GL_FRAGMENT);
	cgGLSetOptimalOptions(*cgFragmentProfile);

	*cgVertexProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/ShadowMapping/resource/cg/vp_shadowMapping.cg",
		*cgVertexProfile,
		"vp_shadowMapping", 
		NULL );

	*cgFragmentProgram = cgCreateProgramFromFile(cgContext,
		CG_SOURCE,
		"../../Shader/ShadowMapping/resource/cg/fp_shadowMapping.cg",
		*cgFragmentProfile,
		"fp_shadowMapping", 
		NULL );


	cgGLLoadProgram(*cgVertexProgram);
	cgGLLoadProgram(*cgFragmentProgram);


	cgModelViewProj		= cgGetNamedParameter(*cgVertexProgram, "modelViewProj");
	cgModelView			= cgGetNamedParameter(*cgVertexProgram, "modelView");
	cgModelViewIT   	= cgGetNamedParameter(*cgVertexProgram, "modelViewIT");	
	cgEyePosition       = cgGetNamedParameter(*cgVertexProgram, "eyePosition");
	cgLightPosition     = cgGetNamedParameter(*cgVertexProgram, "lightPosition");

	if(!cgModelViewProj ||
		!cgModelView ||
		!cgModelViewIT ||
		!cgEyePosition ||
		!cgLightPosition
		)
	{
		printf("Unable to retrieve program parameters, exiting...\n");
		exit(0);
	}
}

void ShadowMapping::setShaderParameters(Vector3f *lightPosition, Vector3f cameraPosition)
{	
	cgGLSetStateMatrixParameter(cgModelViewProj,
								CG_GL_MODELVIEW_PROJECTION_MATRIX,
								CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelView,
								CG_GL_MODELVIEW_MATRIX,
								CG_GL_MATRIX_IDENTITY);
	cgGLSetStateMatrixParameter(cgModelViewIT,
								CG_GL_MODELVIEW_MATRIX,
								CG_GL_MATRIX_INVERSE_TRANSPOSE);

	cgGLSetParameter3f(cgEyePosition, cameraPosition.x(), cameraPosition.y(), cameraPosition.z());
	cgGLSetParameter3f(cgLightPosition, lightPosition->x(), lightPosition->y(), lightPosition->z());
}

void ShadowMapping::drawShadowMap(void)
{

}
